# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_SET_SPACE_METADATA = Metadata(
    id="8a2d5fe6bb459e834b704302eaed7589dbed6387.boutiques",
    name="volume-set-space",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


VolumeSetSpacePlumbParameters = typing.TypedDict('VolumeSetSpacePlumbParameters', {
    "@type": typing.NotRequired[typing.Literal["plumb"]],
    "axis_order": str,
    "x_spacing": float,
    "y_spacing": float,
    "z_spacing": float,
    "x_offset": float,
    "y_offset": float,
    "z_offset": float,
})
VolumeSetSpacePlumbParametersTagged = typing.TypedDict('VolumeSetSpacePlumbParametersTagged', {
    "@type": typing.Literal["plumb"],
    "axis_order": str,
    "x_spacing": float,
    "y_spacing": float,
    "z_spacing": float,
    "x_offset": float,
    "y_offset": float,
    "z_offset": float,
})


VolumeSetSpaceSformParameters = typing.TypedDict('VolumeSetSpaceSformParameters', {
    "@type": typing.NotRequired[typing.Literal["sform"]],
    "xi_spacing": float,
    "xj_spacing": float,
    "xk_spacing": float,
    "x_offset": float,
    "yi_spacing": float,
    "yj_spacing": float,
    "yk_spacing": float,
    "y_offset": float,
    "zi_spacing": float,
    "zj_spacing": float,
    "zk_spacing": float,
    "z_offset": float,
})
VolumeSetSpaceSformParametersTagged = typing.TypedDict('VolumeSetSpaceSformParametersTagged', {
    "@type": typing.Literal["sform"],
    "xi_spacing": float,
    "xj_spacing": float,
    "xk_spacing": float,
    "x_offset": float,
    "yi_spacing": float,
    "yj_spacing": float,
    "yk_spacing": float,
    "y_offset": float,
    "zi_spacing": float,
    "zj_spacing": float,
    "zk_spacing": float,
    "z_offset": float,
})


VolumeSetSpaceFileParameters = typing.TypedDict('VolumeSetSpaceFileParameters', {
    "@type": typing.NotRequired[typing.Literal["file"]],
    "volume_ref": str,
    "opt_ignore_dims": bool,
})
VolumeSetSpaceFileParametersTagged = typing.TypedDict('VolumeSetSpaceFileParametersTagged', {
    "@type": typing.Literal["file"],
    "volume_ref": str,
    "opt_ignore_dims": bool,
})


VolumeSetSpaceParameters = typing.TypedDict('VolumeSetSpaceParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/volume-set-space"]],
    "volume_in": InputPathType,
    "volume_out": str,
    "plumb": typing.NotRequired[VolumeSetSpacePlumbParameters | None],
    "sform": typing.NotRequired[VolumeSetSpaceSformParameters | None],
    "file": typing.NotRequired[VolumeSetSpaceFileParameters | None],
})
VolumeSetSpaceParametersTagged = typing.TypedDict('VolumeSetSpaceParametersTagged', {
    "@type": typing.Literal["workbench/volume-set-space"],
    "volume_in": InputPathType,
    "volume_out": str,
    "plumb": typing.NotRequired[VolumeSetSpacePlumbParameters | None],
    "sform": typing.NotRequired[VolumeSetSpaceSformParameters | None],
    "file": typing.NotRequired[VolumeSetSpaceFileParameters | None],
})


def volume_set_space_plumb_params(
    axis_order: str,
    x_spacing: float,
    y_spacing: float,
    z_spacing: float,
    x_offset: float,
    y_offset: float,
    z_offset: float,
) -> VolumeSetSpacePlumbParametersTagged:
    """
    Build parameters.
    
    Args:
        axis_order: a string like 'XYZ' that specifies which index is along\
            which spatial dimension.
        x_spacing: change in x-coordinate from incrementing the relevant index.
        y_spacing: change in y-coordinate from incrementing the relevant index.
        z_spacing: change in z-coordinate from incrementing the relevant index.
        x_offset: the x-coordinate of the first voxel.
        y_offset: the y-coordinate of the first voxel.
        z_offset: the z-coordinate of the first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "plumb",
        "axis_order": axis_order,
        "x_spacing": x_spacing,
        "y_spacing": y_spacing,
        "z_spacing": z_spacing,
        "x_offset": x_offset,
        "y_offset": y_offset,
        "z_offset": z_offset,
    }
    return params


def volume_set_space_plumb_cargs(
    params: VolumeSetSpacePlumbParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-plumb")
    cargs.append(params.get("axis_order", None))
    cargs.append(str(params.get("x_spacing", None)))
    cargs.append(str(params.get("y_spacing", None)))
    cargs.append(str(params.get("z_spacing", None)))
    cargs.append(str(params.get("x_offset", None)))
    cargs.append(str(params.get("y_offset", None)))
    cargs.append(str(params.get("z_offset", None)))
    return cargs


def volume_set_space_sform_params(
    xi_spacing: float,
    xj_spacing: float,
    xk_spacing: float,
    x_offset: float,
    yi_spacing: float,
    yj_spacing: float,
    yk_spacing: float,
    y_offset: float,
    zi_spacing: float,
    zj_spacing: float,
    zk_spacing: float,
    z_offset: float,
) -> VolumeSetSpaceSformParametersTagged:
    """
    Build parameters.
    
    Args:
        xi_spacing: increase in x coordinate from incrementing the i index.
        xj_spacing: increase in x coordinate from incrementing the j index.
        xk_spacing: increase in x coordinate from incrementing the k index.
        x_offset: x coordinate of first voxel.
        yi_spacing: increase in y coordinate from incrementing the i index.
        yj_spacing: increase in y coordinate from incrementing the j index.
        yk_spacing: increase in y coordinate from incrementing the k index.
        y_offset: y coordinate of first voxel.
        zi_spacing: increase in z coordinate from incrementing the i index.
        zj_spacing: increase in z coordinate from incrementing the j index.
        zk_spacing: increase in z coordinate from incrementing the k index.
        z_offset: z coordinate of first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "sform",
        "xi_spacing": xi_spacing,
        "xj_spacing": xj_spacing,
        "xk_spacing": xk_spacing,
        "x_offset": x_offset,
        "yi_spacing": yi_spacing,
        "yj_spacing": yj_spacing,
        "yk_spacing": yk_spacing,
        "y_offset": y_offset,
        "zi_spacing": zi_spacing,
        "zj_spacing": zj_spacing,
        "zk_spacing": zk_spacing,
        "z_offset": z_offset,
    }
    return params


def volume_set_space_sform_cargs(
    params: VolumeSetSpaceSformParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-sform")
    cargs.append(str(params.get("xi_spacing", None)))
    cargs.append(str(params.get("xj_spacing", None)))
    cargs.append(str(params.get("xk_spacing", None)))
    cargs.append(str(params.get("x_offset", None)))
    cargs.append(str(params.get("yi_spacing", None)))
    cargs.append(str(params.get("yj_spacing", None)))
    cargs.append(str(params.get("yk_spacing", None)))
    cargs.append(str(params.get("y_offset", None)))
    cargs.append(str(params.get("zi_spacing", None)))
    cargs.append(str(params.get("zj_spacing", None)))
    cargs.append(str(params.get("zk_spacing", None)))
    cargs.append(str(params.get("z_offset", None)))
    return cargs


def volume_set_space_file_params(
    volume_ref: str,
    opt_ignore_dims: bool = False,
) -> VolumeSetSpaceFileParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_ref: volume file to use for reference space.
        opt_ignore_dims: copy the spacing info even if the dimensions don't\
            match.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "file",
        "volume_ref": volume_ref,
        "opt_ignore_dims": opt_ignore_dims,
    }
    return params


def volume_set_space_file_cargs(
    params: VolumeSetSpaceFileParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-file")
    cargs.append(params.get("volume_ref", None))
    if params.get("opt_ignore_dims", False):
        cargs.append("-ignore-dims")
    return cargs


class VolumeSetSpaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeSetSpaceParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_set_space_params(
    volume_in: InputPathType,
    volume_out: str,
    plumb: VolumeSetSpacePlumbParameters | None = None,
    sform: VolumeSetSpaceSformParameters | None = None,
    file: VolumeSetSpaceFileParameters | None = None,
) -> VolumeSetSpaceParametersTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        volume_out: output - the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
        file: copy spacing info from volume file with matching dimensions.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-set-space",
        "volume_in": volume_in,
        "volume_out": volume_out,
    }
    if plumb is not None:
        params["plumb"] = plumb
    if sform is not None:
        params["sform"] = sform
    if file is not None:
        params["file"] = file
    return params


def volume_set_space_cargs(
    params: VolumeSetSpaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-set-space")
    cargs.append(execution.input_file(params.get("volume_in", None)))
    cargs.append(params.get("volume_out", None))
    if params.get("plumb", None) is not None:
        cargs.extend(volume_set_space_plumb_cargs(params.get("plumb", None), execution))
    if params.get("sform", None) is not None:
        cargs.extend(volume_set_space_sform_cargs(params.get("sform", None), execution))
    if params.get("file", None) is not None:
        cargs.extend(volume_set_space_file_cargs(params.get("file", None), execution))
    return cargs


def volume_set_space_outputs(
    params: VolumeSetSpaceParameters,
    execution: Execution,
) -> VolumeSetSpaceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeSetSpaceOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_set_space_execute(
    params: VolumeSetSpaceParameters,
    runner: Runner | None = None,
) -> VolumeSetSpaceOutputs:
    """
    volume-set-space
    
    Change volume space information.
    
    Writes a copy of the volume file, with the spacing information changed as
    specified. No reordering of the voxel data occurs, see -volume-reorient to
    change the volume indexing order and reorder the voxels to match. Exactly
    one of -plumb, -sform, or -file must be specified.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSetSpaceOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_SET_SPACE_METADATA)
    params = execution.params(params)
    cargs = volume_set_space_cargs(params, execution)
    ret = volume_set_space_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_set_space(
    volume_in: InputPathType,
    volume_out: str,
    plumb: VolumeSetSpacePlumbParameters | None = None,
    sform: VolumeSetSpaceSformParameters | None = None,
    file: VolumeSetSpaceFileParameters | None = None,
    runner: Runner | None = None,
) -> VolumeSetSpaceOutputs:
    """
    volume-set-space
    
    Change volume space information.
    
    Writes a copy of the volume file, with the spacing information changed as
    specified. No reordering of the voxel data occurs, see -volume-reorient to
    change the volume indexing order and reorder the voxels to match. Exactly
    one of -plumb, -sform, or -file must be specified.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        volume_in: the input volume.
        volume_out: output - the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
        file: copy spacing info from volume file with matching dimensions.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSetSpaceOutputs`).
    """
    params = volume_set_space_params(
        volume_in=volume_in,
        volume_out=volume_out,
        plumb=plumb,
        sform=sform,
        file=file,
    )
    return volume_set_space_execute(params, runner)


__all__ = [
    "VOLUME_SET_SPACE_METADATA",
    "VolumeSetSpaceOutputs",
    "volume_set_space",
    "volume_set_space_execute",
    "volume_set_space_file_params",
    "volume_set_space_params",
    "volume_set_space_plumb_params",
    "volume_set_space_sform_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_DILATE_METADATA = Metadata(
    id="8022de13973d835f4159883ba6f78151a9f68d44.boutiques",
    name="cifti-dilate",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiDilateLeftSurfaceParameters = typing.TypedDict('CiftiDilateLeftSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["left_surface"]],
    "surface": InputPathType,
    "opt_left_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateLeftSurfaceParametersTagged = typing.TypedDict('CiftiDilateLeftSurfaceParametersTagged', {
    "@type": typing.Literal["left_surface"],
    "surface": InputPathType,
    "opt_left_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiDilateRightSurfaceParameters = typing.TypedDict('CiftiDilateRightSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["right_surface"]],
    "surface": InputPathType,
    "opt_right_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateRightSurfaceParametersTagged = typing.TypedDict('CiftiDilateRightSurfaceParametersTagged', {
    "@type": typing.Literal["right_surface"],
    "surface": InputPathType,
    "opt_right_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiDilateCerebellumSurfaceParameters = typing.TypedDict('CiftiDilateCerebellumSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_surface"]],
    "surface": InputPathType,
    "opt_cerebellum_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateCerebellumSurfaceParametersTagged = typing.TypedDict('CiftiDilateCerebellumSurfaceParametersTagged', {
    "@type": typing.Literal["cerebellum_surface"],
    "surface": InputPathType,
    "opt_cerebellum_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiDilateParameters = typing.TypedDict('CiftiDilateParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-dilate"]],
    "cifti_in": InputPathType,
    "direction": str,
    "surface_distance": float,
    "volume_distance": float,
    "cifti_out": str,
    "left_surface": typing.NotRequired[CiftiDilateLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiDilateRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiDilateCerebellumSurfaceParameters | None],
    "opt_bad_brainordinate_roi_roi_cifti": typing.NotRequired[InputPathType | None],
    "opt_nearest": bool,
    "opt_merged_volume": bool,
    "opt_legacy_mode": bool,
})
CiftiDilateParametersTagged = typing.TypedDict('CiftiDilateParametersTagged', {
    "@type": typing.Literal["workbench/cifti-dilate"],
    "cifti_in": InputPathType,
    "direction": str,
    "surface_distance": float,
    "volume_distance": float,
    "cifti_out": str,
    "left_surface": typing.NotRequired[CiftiDilateLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiDilateRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiDilateCerebellumSurfaceParameters | None],
    "opt_bad_brainordinate_roi_roi_cifti": typing.NotRequired[InputPathType | None],
    "opt_nearest": bool,
    "opt_merged_volume": bool,
    "opt_legacy_mode": bool,
})


def cifti_dilate_left_surface_params(
    surface: InputPathType,
    opt_left_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiDilateLeftSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        opt_left_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the left surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_surface",
        "surface": surface,
    }
    if opt_left_corrected_areas_area_metric is not None:
        params["opt_left_corrected_areas_area_metric"] = opt_left_corrected_areas_area_metric
    return params


def cifti_dilate_left_surface_cargs(
    params: CiftiDilateLeftSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_left_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-left-corrected-areas",
            execution.input_file(params.get("opt_left_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_dilate_right_surface_params(
    surface: InputPathType,
    opt_right_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiDilateRightSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        opt_right_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the right surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_surface",
        "surface": surface,
    }
    if opt_right_corrected_areas_area_metric is not None:
        params["opt_right_corrected_areas_area_metric"] = opt_right_corrected_areas_area_metric
    return params


def cifti_dilate_right_surface_cargs(
    params: CiftiDilateRightSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_right_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-right-corrected-areas",
            execution.input_file(params.get("opt_right_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_dilate_cerebellum_surface_params(
    surface: InputPathType,
    opt_cerebellum_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiDilateCerebellumSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        opt_cerebellum_corrected_areas_area_metric: vertex areas to use instead\
            of computing them from the cerebellum surface: the corrected vertex\
            areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_surface",
        "surface": surface,
    }
    if opt_cerebellum_corrected_areas_area_metric is not None:
        params["opt_cerebellum_corrected_areas_area_metric"] = opt_cerebellum_corrected_areas_area_metric
    return params


def cifti_dilate_cerebellum_surface_cargs(
    params: CiftiDilateCerebellumSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_cerebellum_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-cerebellum-corrected-areas",
            execution.input_file(params.get("opt_cerebellum_corrected_areas_area_metric", None))
        ])
    return cargs


class CiftiDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiDilateParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_dilate_params(
    cifti_in: InputPathType,
    direction: str,
    surface_distance: float,
    volume_distance: float,
    cifti_out: str,
    left_surface: CiftiDilateLeftSurfaceParameters | None = None,
    right_surface: CiftiDilateRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiDilateCerebellumSurfaceParameters | None = None,
    opt_bad_brainordinate_roi_roi_cifti: InputPathType | None = None,
    opt_nearest: bool = False,
    opt_merged_volume: bool = False,
    opt_legacy_mode: bool = False,
) -> CiftiDilateParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        direction: which dimension to dilate along, ROW or COLUMN.
        surface_distance: the distance to dilate on surfaces, in mm.
        volume_distance: the distance to dilate in the volume, in mm.
        cifti_out: the output cifti file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_bad_brainordinate_roi_roi_cifti: specify an roi of brainordinates\
            to overwrite, rather than zeros: cifti dscalar or dtseries file,\
            positive values denote brainordinates to have their values replaced.
        opt_nearest: use nearest good value instead of a weighted average.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        opt_legacy_mode: use the math from v1.3.2 and earlier for weighted\
            dilation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-dilate",
        "cifti_in": cifti_in,
        "direction": direction,
        "surface_distance": surface_distance,
        "volume_distance": volume_distance,
        "cifti_out": cifti_out,
        "opt_nearest": opt_nearest,
        "opt_merged_volume": opt_merged_volume,
        "opt_legacy_mode": opt_legacy_mode,
    }
    if left_surface is not None:
        params["left_surface"] = left_surface
    if right_surface is not None:
        params["right_surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum_surface"] = cerebellum_surface
    if opt_bad_brainordinate_roi_roi_cifti is not None:
        params["opt_bad_brainordinate_roi_roi_cifti"] = opt_bad_brainordinate_roi_roi_cifti
    return params


def cifti_dilate_cargs(
    params: CiftiDilateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-dilate")
    cargs.append(execution.input_file(params.get("cifti_in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(str(params.get("surface_distance", None)))
    cargs.append(str(params.get("volume_distance", None)))
    cargs.append(params.get("cifti_out", None))
    if params.get("left_surface", None) is not None:
        cargs.extend(cifti_dilate_left_surface_cargs(params.get("left_surface", None), execution))
    if params.get("right_surface", None) is not None:
        cargs.extend(cifti_dilate_right_surface_cargs(params.get("right_surface", None), execution))
    if params.get("cerebellum_surface", None) is not None:
        cargs.extend(cifti_dilate_cerebellum_surface_cargs(params.get("cerebellum_surface", None), execution))
    if params.get("opt_bad_brainordinate_roi_roi_cifti", None) is not None:
        cargs.extend([
            "-bad-brainordinate-roi",
            execution.input_file(params.get("opt_bad_brainordinate_roi_roi_cifti", None))
        ])
    if params.get("opt_nearest", False):
        cargs.append("-nearest")
    if params.get("opt_merged_volume", False):
        cargs.append("-merged-volume")
    if params.get("opt_legacy_mode", False):
        cargs.append("-legacy-mode")
    return cargs


def cifti_dilate_outputs(
    params: CiftiDilateParameters,
    execution: Execution,
) -> CiftiDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiDilateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_dilate_execute(
    params: CiftiDilateParameters,
    runner: Runner | None = None,
) -> CiftiDilateOutputs:
    """
    cifti-dilate
    
    Dilate a cifti file.
    
    For all data values designated as bad, if they neighbor a good value or are
    within the specified distance of a good value in the same kind of model,
    replace the value with a distance weighted average of nearby good values,
    otherwise set the value to zero. If -nearest is specified, it will use the
    value from the closest good value within range instead of a weighted
    average. When the input file contains label data, nearest dilation is used
    on the surface, and weighted popularity is used in the volume.
    
    The -*-corrected-areas options are intended for dilating on group average
    surfaces, but it is only an approximate correction for the reduction of
    structure in a group average surface.
    
    If -bad-brainordinate-roi is specified, all values, including those with
    value zero, are good, except for locations with a positive value in the ROI.
    If it is not specified, only values equal to zero are bad.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiDilateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_DILATE_METADATA)
    params = execution.params(params)
    cargs = cifti_dilate_cargs(params, execution)
    ret = cifti_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_dilate(
    cifti_in: InputPathType,
    direction: str,
    surface_distance: float,
    volume_distance: float,
    cifti_out: str,
    left_surface: CiftiDilateLeftSurfaceParameters | None = None,
    right_surface: CiftiDilateRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiDilateCerebellumSurfaceParameters | None = None,
    opt_bad_brainordinate_roi_roi_cifti: InputPathType | None = None,
    opt_nearest: bool = False,
    opt_merged_volume: bool = False,
    opt_legacy_mode: bool = False,
    runner: Runner | None = None,
) -> CiftiDilateOutputs:
    """
    cifti-dilate
    
    Dilate a cifti file.
    
    For all data values designated as bad, if they neighbor a good value or are
    within the specified distance of a good value in the same kind of model,
    replace the value with a distance weighted average of nearby good values,
    otherwise set the value to zero. If -nearest is specified, it will use the
    value from the closest good value within range instead of a weighted
    average. When the input file contains label data, nearest dilation is used
    on the surface, and weighted popularity is used in the volume.
    
    The -*-corrected-areas options are intended for dilating on group average
    surfaces, but it is only an approximate correction for the reduction of
    structure in a group average surface.
    
    If -bad-brainordinate-roi is specified, all values, including those with
    value zero, are good, except for locations with a positive value in the ROI.
    If it is not specified, only values equal to zero are bad.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_in: the input cifti file.
        direction: which dimension to dilate along, ROW or COLUMN.
        surface_distance: the distance to dilate on surfaces, in mm.
        volume_distance: the distance to dilate in the volume, in mm.
        cifti_out: the output cifti file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_bad_brainordinate_roi_roi_cifti: specify an roi of brainordinates\
            to overwrite, rather than zeros: cifti dscalar or dtseries file,\
            positive values denote brainordinates to have their values replaced.
        opt_nearest: use nearest good value instead of a weighted average.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        opt_legacy_mode: use the math from v1.3.2 and earlier for weighted\
            dilation.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiDilateOutputs`).
    """
    params = cifti_dilate_params(
        cifti_in=cifti_in,
        direction=direction,
        surface_distance=surface_distance,
        volume_distance=volume_distance,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        opt_bad_brainordinate_roi_roi_cifti=opt_bad_brainordinate_roi_roi_cifti,
        opt_nearest=opt_nearest,
        opt_merged_volume=opt_merged_volume,
        opt_legacy_mode=opt_legacy_mode,
    )
    return cifti_dilate_execute(params, runner)


__all__ = [
    "CIFTI_DILATE_METADATA",
    "CiftiDilateOutputs",
    "cifti_dilate",
    "cifti_dilate_cerebellum_surface_params",
    "cifti_dilate_execute",
    "cifti_dilate_left_surface_params",
    "cifti_dilate_params",
    "cifti_dilate_right_surface_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_INFLATION_METADATA = Metadata(
    id="8d8e7af8d80a5930fe793979c523d20e13520896.boutiques",
    name="surface-inflation",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


SurfaceInflationParameters = typing.TypedDict('SurfaceInflationParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/surface-inflation"]],
    "anatomical_surface_in": InputPathType,
    "surface_in": InputPathType,
    "number_of_smoothing_cycles": int,
    "smoothing_strength": float,
    "smoothing_iterations": int,
    "inflation_factor": float,
    "surface_out": str,
})
SurfaceInflationParametersTagged = typing.TypedDict('SurfaceInflationParametersTagged', {
    "@type": typing.Literal["workbench/surface-inflation"],
    "anatomical_surface_in": InputPathType,
    "surface_in": InputPathType,
    "number_of_smoothing_cycles": int,
    "smoothing_strength": float,
    "smoothing_iterations": int,
    "inflation_factor": float,
    "surface_out": str,
})


class SurfaceInflationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceInflationParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """output surface file"""


def surface_inflation_params(
    anatomical_surface_in: InputPathType,
    surface_in: InputPathType,
    number_of_smoothing_cycles: int,
    smoothing_strength: float,
    smoothing_iterations: int,
    inflation_factor: float,
    surface_out: str,
) -> SurfaceInflationParametersTagged:
    """
    Build parameters.
    
    Args:
        anatomical_surface_in: the anatomical surface.
        surface_in: the surface file to inflate.
        number_of_smoothing_cycles: number of smoothing cycles.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        inflation_factor: inflation factor.
        surface_out: output surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-inflation",
        "anatomical_surface_in": anatomical_surface_in,
        "surface_in": surface_in,
        "number_of_smoothing_cycles": number_of_smoothing_cycles,
        "smoothing_strength": smoothing_strength,
        "smoothing_iterations": smoothing_iterations,
        "inflation_factor": inflation_factor,
        "surface_out": surface_out,
    }
    return params


def surface_inflation_cargs(
    params: SurfaceInflationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-surface-inflation")
    cargs.append(execution.input_file(params.get("anatomical_surface_in", None)))
    cargs.append(execution.input_file(params.get("surface_in", None)))
    cargs.append(str(params.get("number_of_smoothing_cycles", None)))
    cargs.append(str(params.get("smoothing_strength", None)))
    cargs.append(str(params.get("smoothing_iterations", None)))
    cargs.append(str(params.get("inflation_factor", None)))
    cargs.append(params.get("surface_out", None))
    return cargs


def surface_inflation_outputs(
    params: SurfaceInflationParameters,
    execution: Execution,
) -> SurfaceInflationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceInflationOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface_out", None)),
    )
    return ret


def surface_inflation_execute(
    params: SurfaceInflationParameters,
    runner: Runner | None = None,
) -> SurfaceInflationOutputs:
    """
    surface-inflation
    
    Surface inflation.
    
    Inflate a surface by performing cycles that consist of smoothing followed by
    inflation (to correct shrinkage caused by smoothing).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInflationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_INFLATION_METADATA)
    params = execution.params(params)
    cargs = surface_inflation_cargs(params, execution)
    ret = surface_inflation_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_inflation(
    anatomical_surface_in: InputPathType,
    surface_in: InputPathType,
    number_of_smoothing_cycles: int,
    smoothing_strength: float,
    smoothing_iterations: int,
    inflation_factor: float,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceInflationOutputs:
    """
    surface-inflation
    
    Surface inflation.
    
    Inflate a surface by performing cycles that consist of smoothing followed by
    inflation (to correct shrinkage caused by smoothing).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        anatomical_surface_in: the anatomical surface.
        surface_in: the surface file to inflate.
        number_of_smoothing_cycles: number of smoothing cycles.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        inflation_factor: inflation factor.
        surface_out: output surface file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInflationOutputs`).
    """
    params = surface_inflation_params(
        anatomical_surface_in=anatomical_surface_in,
        surface_in=surface_in,
        number_of_smoothing_cycles=number_of_smoothing_cycles,
        smoothing_strength=smoothing_strength,
        smoothing_iterations=smoothing_iterations,
        inflation_factor=inflation_factor,
        surface_out=surface_out,
    )
    return surface_inflation_execute(params, runner)


__all__ = [
    "SURFACE_INFLATION_METADATA",
    "SurfaceInflationOutputs",
    "surface_inflation",
    "surface_inflation_execute",
    "surface_inflation_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_REGRESSION_METADATA = Metadata(
    id="478fb33cff7af2c028c82b1a87cc021ded62f90d.boutiques",
    name="metric-regression",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


MetricRegressionRemoveParameters = typing.TypedDict('MetricRegressionRemoveParameters', {
    "@type": typing.NotRequired[typing.Literal["remove"]],
    "metric": InputPathType,
    "opt_remove_column_column": typing.NotRequired[str | None],
})
MetricRegressionRemoveParametersTagged = typing.TypedDict('MetricRegressionRemoveParametersTagged', {
    "@type": typing.Literal["remove"],
    "metric": InputPathType,
    "opt_remove_column_column": typing.NotRequired[str | None],
})


MetricRegressionKeepParameters = typing.TypedDict('MetricRegressionKeepParameters', {
    "@type": typing.NotRequired[typing.Literal["keep"]],
    "metric": InputPathType,
    "opt_keep_column_column": typing.NotRequired[str | None],
})
MetricRegressionKeepParametersTagged = typing.TypedDict('MetricRegressionKeepParametersTagged', {
    "@type": typing.Literal["keep"],
    "metric": InputPathType,
    "opt_keep_column_column": typing.NotRequired[str | None],
})


MetricRegressionParameters = typing.TypedDict('MetricRegressionParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/metric-regression"]],
    "metric_in": InputPathType,
    "metric_out": str,
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "remove": typing.NotRequired[list[MetricRegressionRemoveParameters] | None],
    "keep": typing.NotRequired[list[MetricRegressionKeepParameters] | None],
})
MetricRegressionParametersTagged = typing.TypedDict('MetricRegressionParametersTagged', {
    "@type": typing.Literal["workbench/metric-regression"],
    "metric_in": InputPathType,
    "metric_out": str,
    "opt_roi_roi_metric": typing.NotRequired[InputPathType | None],
    "opt_column_column": typing.NotRequired[str | None],
    "remove": typing.NotRequired[list[MetricRegressionRemoveParameters] | None],
    "keep": typing.NotRequired[list[MetricRegressionKeepParameters] | None],
})


def metric_regression_remove_params(
    metric: InputPathType,
    opt_remove_column_column: str | None = None,
) -> MetricRegressionRemoveParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file to use.
        opt_remove_column_column: select a column to use, rather than all: the\
            column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "remove",
        "metric": metric,
    }
    if opt_remove_column_column is not None:
        params["opt_remove_column_column"] = opt_remove_column_column
    return params


def metric_regression_remove_cargs(
    params: MetricRegressionRemoveParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-remove")
    cargs.append(execution.input_file(params.get("metric", None)))
    if params.get("opt_remove_column_column", None) is not None:
        cargs.extend([
            "-remove-column",
            params.get("opt_remove_column_column", None)
        ])
    return cargs


def metric_regression_keep_params(
    metric: InputPathType,
    opt_keep_column_column: str | None = None,
) -> MetricRegressionKeepParametersTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file to use.
        opt_keep_column_column: select a column to use, rather than all: the\
            column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "keep",
        "metric": metric,
    }
    if opt_keep_column_column is not None:
        params["opt_keep_column_column"] = opt_keep_column_column
    return params


def metric_regression_keep_cargs(
    params: MetricRegressionKeepParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-keep")
    cargs.append(execution.input_file(params.get("metric", None)))
    if params.get("opt_keep_column_column", None) is not None:
        cargs.extend([
            "-keep-column",
            params.get("opt_keep_column_column", None)
        ])
    return cargs


class MetricRegressionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricRegressionParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_regression_params(
    metric_in: InputPathType,
    metric_out: str,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    remove: list[MetricRegressionRemoveParameters] | None = None,
    keep: list[MetricRegressionKeepParameters] | None = None,
) -> MetricRegressionParametersTagged:
    """
    Build parameters.
    
    Args:
        metric_in: the metric to regress from.
        metric_out: the output metric.
        opt_roi_roi_metric: only regress inside an roi: the area to use for\
            regression, as a metric.
        opt_column_column: select a single column to regress from: the column\
            number or name.
        remove: specify a metric to regress out.
        keep: specify a metric to include in regression, but not remove.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-regression",
        "metric_in": metric_in,
        "metric_out": metric_out,
    }
    if opt_roi_roi_metric is not None:
        params["opt_roi_roi_metric"] = opt_roi_roi_metric
    if opt_column_column is not None:
        params["opt_column_column"] = opt_column_column
    if remove is not None:
        params["remove"] = remove
    if keep is not None:
        params["keep"] = keep
    return params


def metric_regression_cargs(
    params: MetricRegressionParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-regression")
    cargs.append(execution.input_file(params.get("metric_in", None)))
    cargs.append(params.get("metric_out", None))
    if params.get("opt_roi_roi_metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("opt_roi_roi_metric", None))
        ])
    if params.get("opt_column_column", None) is not None:
        cargs.extend([
            "-column",
            params.get("opt_column_column", None)
        ])
    if params.get("remove", None) is not None:
        cargs.extend([a for c in [metric_regression_remove_cargs(s, execution) for s in params.get("remove", None)] for a in c])
    if params.get("keep", None) is not None:
        cargs.extend([a for c in [metric_regression_keep_cargs(s, execution) for s in params.get("keep", None)] for a in c])
    return cargs


def metric_regression_outputs(
    params: MetricRegressionParameters,
    execution: Execution,
) -> MetricRegressionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricRegressionOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric_out", None)),
    )
    return ret


def metric_regression_execute(
    params: MetricRegressionParameters,
    runner: Runner | None = None,
) -> MetricRegressionOutputs:
    """
    metric-regression
    
    Regress spatial map out of a metric file.
    
    For each regressor, its mean across the surface is subtracted from its data.
    Each input map is then regressed against these, and a constant term. The
    resulting regressed slopes of all regressors specified with -remove are
    multiplied with their respective regressor maps, and these are subtracted
    from the input map.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRegressionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_REGRESSION_METADATA)
    params = execution.params(params)
    cargs = metric_regression_cargs(params, execution)
    ret = metric_regression_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_regression(
    metric_in: InputPathType,
    metric_out: str,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    remove: list[MetricRegressionRemoveParameters] | None = None,
    keep: list[MetricRegressionKeepParameters] | None = None,
    runner: Runner | None = None,
) -> MetricRegressionOutputs:
    """
    metric-regression
    
    Regress spatial map out of a metric file.
    
    For each regressor, its mean across the surface is subtracted from its data.
    Each input map is then regressed against these, and a constant term. The
    resulting regressed slopes of all regressors specified with -remove are
    multiplied with their respective regressor maps, and these are subtracted
    from the input map.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        metric_in: the metric to regress from.
        metric_out: the output metric.
        opt_roi_roi_metric: only regress inside an roi: the area to use for\
            regression, as a metric.
        opt_column_column: select a single column to regress from: the column\
            number or name.
        remove: specify a metric to regress out.
        keep: specify a metric to include in regression, but not remove.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRegressionOutputs`).
    """
    params = metric_regression_params(
        metric_in=metric_in,
        metric_out=metric_out,
        opt_roi_roi_metric=opt_roi_roi_metric,
        opt_column_column=opt_column_column,
        remove=remove,
        keep=keep,
    )
    return metric_regression_execute(params, runner)


__all__ = [
    "METRIC_REGRESSION_METADATA",
    "MetricRegressionOutputs",
    "metric_regression",
    "metric_regression_execute",
    "metric_regression_keep_params",
    "metric_regression_params",
    "metric_regression_remove_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CORRELATION_GRADIENT_METADATA = Metadata(
    id="d806149417489bcd1887925649598dc5eb308a60.boutiques",
    name="cifti-correlation-gradient",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiCorrelationGradientLeftSurfaceParameters = typing.TypedDict('CiftiCorrelationGradientLeftSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["left_surface"]],
    "surface": InputPathType,
    "opt_left_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientLeftSurfaceParametersTagged = typing.TypedDict('CiftiCorrelationGradientLeftSurfaceParametersTagged', {
    "@type": typing.Literal["left_surface"],
    "surface": InputPathType,
    "opt_left_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiCorrelationGradientRightSurfaceParameters = typing.TypedDict('CiftiCorrelationGradientRightSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["right_surface"]],
    "surface": InputPathType,
    "opt_right_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientRightSurfaceParametersTagged = typing.TypedDict('CiftiCorrelationGradientRightSurfaceParametersTagged', {
    "@type": typing.Literal["right_surface"],
    "surface": InputPathType,
    "opt_right_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiCorrelationGradientCerebellumSurfaceParameters = typing.TypedDict('CiftiCorrelationGradientCerebellumSurfaceParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_surface"]],
    "surface": InputPathType,
    "opt_cerebellum_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientCerebellumSurfaceParametersTagged = typing.TypedDict('CiftiCorrelationGradientCerebellumSurfaceParametersTagged', {
    "@type": typing.Literal["cerebellum_surface"],
    "surface": InputPathType,
    "opt_cerebellum_corrected_areas_area_metric": typing.NotRequired[InputPathType | None],
})


CiftiCorrelationGradientDoubleCorrelationParameters = typing.TypedDict('CiftiCorrelationGradientDoubleCorrelationParameters', {
    "@type": typing.NotRequired[typing.Literal["double_correlation"]],
    "opt_fisher_z_first": bool,
    "opt_no_demean_first": bool,
    "opt_covariance_first": bool,
})
CiftiCorrelationGradientDoubleCorrelationParametersTagged = typing.TypedDict('CiftiCorrelationGradientDoubleCorrelationParametersTagged', {
    "@type": typing.Literal["double_correlation"],
    "opt_fisher_z_first": bool,
    "opt_no_demean_first": bool,
    "opt_covariance_first": bool,
})


CiftiCorrelationGradientParameters = typing.TypedDict('CiftiCorrelationGradientParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-correlation-gradient"]],
    "cifti": InputPathType,
    "cifti_out": str,
    "left_surface": typing.NotRequired[CiftiCorrelationGradientLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiCorrelationGradientRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiCorrelationGradientCerebellumSurfaceParameters | None],
    "opt_surface_presmooth_surface_kernel": typing.NotRequired[float | None],
    "opt_volume_presmooth_volume_kernel": typing.NotRequired[float | None],
    "opt_presmooth_fwhm": bool,
    "opt_undo_fisher_z": bool,
    "opt_fisher_z": bool,
    "opt_surface_exclude_distance": typing.NotRequired[float | None],
    "opt_volume_exclude_distance": typing.NotRequired[float | None],
    "opt_covariance": bool,
    "opt_mem_limit_limit_gb": typing.NotRequired[float | None],
    "double_correlation": typing.NotRequired[CiftiCorrelationGradientDoubleCorrelationParameters | None],
})
CiftiCorrelationGradientParametersTagged = typing.TypedDict('CiftiCorrelationGradientParametersTagged', {
    "@type": typing.Literal["workbench/cifti-correlation-gradient"],
    "cifti": InputPathType,
    "cifti_out": str,
    "left_surface": typing.NotRequired[CiftiCorrelationGradientLeftSurfaceParameters | None],
    "right_surface": typing.NotRequired[CiftiCorrelationGradientRightSurfaceParameters | None],
    "cerebellum_surface": typing.NotRequired[CiftiCorrelationGradientCerebellumSurfaceParameters | None],
    "opt_surface_presmooth_surface_kernel": typing.NotRequired[float | None],
    "opt_volume_presmooth_volume_kernel": typing.NotRequired[float | None],
    "opt_presmooth_fwhm": bool,
    "opt_undo_fisher_z": bool,
    "opt_fisher_z": bool,
    "opt_surface_exclude_distance": typing.NotRequired[float | None],
    "opt_volume_exclude_distance": typing.NotRequired[float | None],
    "opt_covariance": bool,
    "opt_mem_limit_limit_gb": typing.NotRequired[float | None],
    "double_correlation": typing.NotRequired[CiftiCorrelationGradientDoubleCorrelationParameters | None],
})


def cifti_correlation_gradient_left_surface_params(
    surface: InputPathType,
    opt_left_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientLeftSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        opt_left_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the left surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_surface",
        "surface": surface,
    }
    if opt_left_corrected_areas_area_metric is not None:
        params["opt_left_corrected_areas_area_metric"] = opt_left_corrected_areas_area_metric
    return params


def cifti_correlation_gradient_left_surface_cargs(
    params: CiftiCorrelationGradientLeftSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_left_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-left-corrected-areas",
            execution.input_file(params.get("opt_left_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_correlation_gradient_right_surface_params(
    surface: InputPathType,
    opt_right_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientRightSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        opt_right_corrected_areas_area_metric: vertex areas to use instead of\
            computing them from the right surface: the corrected vertex areas, as a\
            metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_surface",
        "surface": surface,
    }
    if opt_right_corrected_areas_area_metric is not None:
        params["opt_right_corrected_areas_area_metric"] = opt_right_corrected_areas_area_metric
    return params


def cifti_correlation_gradient_right_surface_cargs(
    params: CiftiCorrelationGradientRightSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_right_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-right-corrected-areas",
            execution.input_file(params.get("opt_right_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_correlation_gradient_cerebellum_surface_params(
    surface: InputPathType,
    opt_cerebellum_corrected_areas_area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientCerebellumSurfaceParametersTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        opt_cerebellum_corrected_areas_area_metric: vertex areas to use instead\
            of computing them from the cerebellum surface: the corrected vertex\
            areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_surface",
        "surface": surface,
    }
    if opt_cerebellum_corrected_areas_area_metric is not None:
        params["opt_cerebellum_corrected_areas_area_metric"] = opt_cerebellum_corrected_areas_area_metric
    return params


def cifti_correlation_gradient_cerebellum_surface_cargs(
    params: CiftiCorrelationGradientCerebellumSurfaceParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-surface")
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("opt_cerebellum_corrected_areas_area_metric", None) is not None:
        cargs.extend([
            "-cerebellum-corrected-areas",
            execution.input_file(params.get("opt_cerebellum_corrected_areas_area_metric", None))
        ])
    return cargs


def cifti_correlation_gradient_double_correlation_params(
    opt_fisher_z_first: bool = False,
    opt_no_demean_first: bool = False,
    opt_covariance_first: bool = False,
) -> CiftiCorrelationGradientDoubleCorrelationParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_fisher_z_first: after the FIRST correlation, apply fisher small z\
            transform (ie, artanh).
        opt_no_demean_first: instead of correlation for the FIRST operation, do\
            dot product of rows, then normalize by diagonal.
        opt_covariance_first: instead of correlation for the FIRST operation,\
            compute covariance.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "double_correlation",
        "opt_fisher_z_first": opt_fisher_z_first,
        "opt_no_demean_first": opt_no_demean_first,
        "opt_covariance_first": opt_covariance_first,
    }
    return params


def cifti_correlation_gradient_double_correlation_cargs(
    params: CiftiCorrelationGradientDoubleCorrelationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-double-correlation")
    if params.get("opt_fisher_z_first", False):
        cargs.append("-fisher-z-first")
    if params.get("opt_no_demean_first", False):
        cargs.append("-no-demean-first")
    if params.get("opt_covariance_first", False):
        cargs.append("-covariance-first")
    return cargs


class CiftiCorrelationGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCorrelationGradientParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_correlation_gradient_params(
    cifti: InputPathType,
    cifti_out: str,
    left_surface: CiftiCorrelationGradientLeftSurfaceParameters | None = None,
    right_surface: CiftiCorrelationGradientRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiCorrelationGradientCerebellumSurfaceParameters | None = None,
    opt_surface_presmooth_surface_kernel: float | None = None,
    opt_volume_presmooth_volume_kernel: float | None = None,
    opt_presmooth_fwhm: bool = False,
    opt_undo_fisher_z: bool = False,
    opt_fisher_z: bool = False,
    opt_surface_exclude_distance: float | None = None,
    opt_volume_exclude_distance: float | None = None,
    opt_covariance: bool = False,
    opt_mem_limit_limit_gb: float | None = None,
    double_correlation: CiftiCorrelationGradientDoubleCorrelationParameters | None = None,
) -> CiftiCorrelationGradientParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti.
        cifti_out: the output cifti.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_surface_presmooth_surface_kernel: smooth on the surface before\
            computing the gradient: the size of the gaussian surface smoothing\
            kernel in mm, as sigma by default.
        opt_volume_presmooth_volume_kernel: smooth the volume before computing\
            the gradient: the size of the gaussian volume smoothing kernel in mm,\
            as sigma by default.
        opt_presmooth_fwhm: smoothing kernel sizes are FWHM, not sigma.
        opt_undo_fisher_z: apply the inverse fisher small z transform to the\
            input.
        opt_fisher_z: apply the fisher small z transform to the correlations\
            before taking the gradient.
        opt_surface_exclude_distance: exclude vertices near each seed vertex\
            from computation: geodesic distance from seed vertex for the exclusion\
            zone, in mm.
        opt_volume_exclude_distance: exclude voxels near each seed voxel from\
            computation: distance from seed voxel for the exclusion zone, in mm.
        opt_covariance: compute covariance instead of correlation.
        opt_mem_limit_limit_gb: restrict memory usage: memory limit in\
            gigabytes.
        double_correlation: do two correlations before taking the gradient.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-correlation-gradient",
        "cifti": cifti,
        "cifti_out": cifti_out,
        "opt_presmooth_fwhm": opt_presmooth_fwhm,
        "opt_undo_fisher_z": opt_undo_fisher_z,
        "opt_fisher_z": opt_fisher_z,
        "opt_covariance": opt_covariance,
    }
    if left_surface is not None:
        params["left_surface"] = left_surface
    if right_surface is not None:
        params["right_surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum_surface"] = cerebellum_surface
    if opt_surface_presmooth_surface_kernel is not None:
        params["opt_surface_presmooth_surface_kernel"] = opt_surface_presmooth_surface_kernel
    if opt_volume_presmooth_volume_kernel is not None:
        params["opt_volume_presmooth_volume_kernel"] = opt_volume_presmooth_volume_kernel
    if opt_surface_exclude_distance is not None:
        params["opt_surface_exclude_distance"] = opt_surface_exclude_distance
    if opt_volume_exclude_distance is not None:
        params["opt_volume_exclude_distance"] = opt_volume_exclude_distance
    if opt_mem_limit_limit_gb is not None:
        params["opt_mem_limit_limit_gb"] = opt_mem_limit_limit_gb
    if double_correlation is not None:
        params["double_correlation"] = double_correlation
    return params


def cifti_correlation_gradient_cargs(
    params: CiftiCorrelationGradientParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-correlation-gradient")
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(params.get("cifti_out", None))
    if params.get("left_surface", None) is not None:
        cargs.extend(cifti_correlation_gradient_left_surface_cargs(params.get("left_surface", None), execution))
    if params.get("right_surface", None) is not None:
        cargs.extend(cifti_correlation_gradient_right_surface_cargs(params.get("right_surface", None), execution))
    if params.get("cerebellum_surface", None) is not None:
        cargs.extend(cifti_correlation_gradient_cerebellum_surface_cargs(params.get("cerebellum_surface", None), execution))
    if params.get("opt_surface_presmooth_surface_kernel", None) is not None:
        cargs.extend([
            "-surface-presmooth",
            str(params.get("opt_surface_presmooth_surface_kernel", None))
        ])
    if params.get("opt_volume_presmooth_volume_kernel", None) is not None:
        cargs.extend([
            "-volume-presmooth",
            str(params.get("opt_volume_presmooth_volume_kernel", None))
        ])
    if params.get("opt_presmooth_fwhm", False):
        cargs.append("-presmooth-fwhm")
    if params.get("opt_undo_fisher_z", False):
        cargs.append("-undo-fisher-z")
    if params.get("opt_fisher_z", False):
        cargs.append("-fisher-z")
    if params.get("opt_surface_exclude_distance", None) is not None:
        cargs.extend([
            "-surface-exclude",
            str(params.get("opt_surface_exclude_distance", None))
        ])
    if params.get("opt_volume_exclude_distance", None) is not None:
        cargs.extend([
            "-volume-exclude",
            str(params.get("opt_volume_exclude_distance", None))
        ])
    if params.get("opt_covariance", False):
        cargs.append("-covariance")
    if params.get("opt_mem_limit_limit_gb", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("opt_mem_limit_limit_gb", None))
        ])
    if params.get("double_correlation", None) is not None:
        cargs.extend(cifti_correlation_gradient_double_correlation_cargs(params.get("double_correlation", None), execution))
    return cargs


def cifti_correlation_gradient_outputs(
    params: CiftiCorrelationGradientParameters,
    execution: Execution,
) -> CiftiCorrelationGradientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCorrelationGradientOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_correlation_gradient_execute(
    params: CiftiCorrelationGradientParameters,
    runner: Runner | None = None,
) -> CiftiCorrelationGradientOutputs:
    """
    cifti-correlation-gradient
    
    Correlate cifti rows and take gradient.
    
    For each structure, compute the correlation of the rows in the structure,
    and take the gradients of the resulting rows, then average them. Memory
    limit does not need to be an integer, you may also specify 0 to use as
    little memory as possible (this may be very slow).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationGradientOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CORRELATION_GRADIENT_METADATA)
    params = execution.params(params)
    cargs = cifti_correlation_gradient_cargs(params, execution)
    ret = cifti_correlation_gradient_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_correlation_gradient(
    cifti: InputPathType,
    cifti_out: str,
    left_surface: CiftiCorrelationGradientLeftSurfaceParameters | None = None,
    right_surface: CiftiCorrelationGradientRightSurfaceParameters | None = None,
    cerebellum_surface: CiftiCorrelationGradientCerebellumSurfaceParameters | None = None,
    opt_surface_presmooth_surface_kernel: float | None = None,
    opt_volume_presmooth_volume_kernel: float | None = None,
    opt_presmooth_fwhm: bool = False,
    opt_undo_fisher_z: bool = False,
    opt_fisher_z: bool = False,
    opt_surface_exclude_distance: float | None = None,
    opt_volume_exclude_distance: float | None = None,
    opt_covariance: bool = False,
    opt_mem_limit_limit_gb: float | None = None,
    double_correlation: CiftiCorrelationGradientDoubleCorrelationParameters | None = None,
    runner: Runner | None = None,
) -> CiftiCorrelationGradientOutputs:
    """
    cifti-correlation-gradient
    
    Correlate cifti rows and take gradient.
    
    For each structure, compute the correlation of the rows in the structure,
    and take the gradients of the resulting rows, then average them. Memory
    limit does not need to be an integer, you may also specify 0 to use as
    little memory as possible (this may be very slow).
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti: the input cifti.
        cifti_out: the output cifti.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        opt_surface_presmooth_surface_kernel: smooth on the surface before\
            computing the gradient: the size of the gaussian surface smoothing\
            kernel in mm, as sigma by default.
        opt_volume_presmooth_volume_kernel: smooth the volume before computing\
            the gradient: the size of the gaussian volume smoothing kernel in mm,\
            as sigma by default.
        opt_presmooth_fwhm: smoothing kernel sizes are FWHM, not sigma.
        opt_undo_fisher_z: apply the inverse fisher small z transform to the\
            input.
        opt_fisher_z: apply the fisher small z transform to the correlations\
            before taking the gradient.
        opt_surface_exclude_distance: exclude vertices near each seed vertex\
            from computation: geodesic distance from seed vertex for the exclusion\
            zone, in mm.
        opt_volume_exclude_distance: exclude voxels near each seed voxel from\
            computation: distance from seed voxel for the exclusion zone, in mm.
        opt_covariance: compute covariance instead of correlation.
        opt_mem_limit_limit_gb: restrict memory usage: memory limit in\
            gigabytes.
        double_correlation: do two correlations before taking the gradient.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationGradientOutputs`).
    """
    params = cifti_correlation_gradient_params(
        cifti=cifti,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        opt_surface_presmooth_surface_kernel=opt_surface_presmooth_surface_kernel,
        opt_volume_presmooth_volume_kernel=opt_volume_presmooth_volume_kernel,
        opt_presmooth_fwhm=opt_presmooth_fwhm,
        opt_undo_fisher_z=opt_undo_fisher_z,
        opt_fisher_z=opt_fisher_z,
        opt_surface_exclude_distance=opt_surface_exclude_distance,
        opt_volume_exclude_distance=opt_volume_exclude_distance,
        opt_covariance=opt_covariance,
        opt_mem_limit_limit_gb=opt_mem_limit_limit_gb,
        double_correlation=double_correlation,
    )
    return cifti_correlation_gradient_execute(params, runner)


__all__ = [
    "CIFTI_CORRELATION_GRADIENT_METADATA",
    "CiftiCorrelationGradientOutputs",
    "cifti_correlation_gradient",
    "cifti_correlation_gradient_cerebellum_surface_params",
    "cifti_correlation_gradient_double_correlation_params",
    "cifti_correlation_gradient_execute",
    "cifti_correlation_gradient_left_surface_params",
    "cifti_correlation_gradient_params",
    "cifti_correlation_gradient_right_surface_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_RESAMPLE_DCONN_MEMORY_METADATA = Metadata(
    id="d729ea969be03fe9e202413b7e489346783a1ac2.boutiques",
    name="cifti-resample-dconn-memory",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiResampleDconnMemoryWeightedParameters = typing.TypedDict('CiftiResampleDconnMemoryWeightedParameters', {
    "@type": typing.NotRequired[typing.Literal["weighted"]],
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_legacy_cutoff": bool,
})
CiftiResampleDconnMemoryWeightedParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryWeightedParametersTagged', {
    "@type": typing.Literal["weighted"],
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_legacy_cutoff": bool,
})


CiftiResampleDconnMemoryVolumePredilateParameters = typing.TypedDict('CiftiResampleDconnMemoryVolumePredilateParameters', {
    "@type": typing.NotRequired[typing.Literal["volume_predilate"]],
    "dilate_mm": float,
    "opt_nearest": bool,
    "weighted": typing.NotRequired[CiftiResampleDconnMemoryWeightedParameters | None],
})
CiftiResampleDconnMemoryVolumePredilateParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryVolumePredilateParametersTagged', {
    "@type": typing.Literal["volume_predilate"],
    "dilate_mm": float,
    "opt_nearest": bool,
    "weighted": typing.NotRequired[CiftiResampleDconnMemoryWeightedParameters | None],
})


CiftiResampleDconnMemoryWeighted1Parameters = typing.TypedDict('CiftiResampleDconnMemoryWeighted1Parameters', {
    "@type": typing.NotRequired[typing.Literal["weighted_1"]],
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_legacy_cutoff": bool,
})
CiftiResampleDconnMemoryWeighted1ParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryWeighted1ParametersTagged', {
    "@type": typing.Literal["weighted_1"],
    "opt_exponent_exponent": typing.NotRequired[float | None],
    "opt_legacy_cutoff": bool,
})


CiftiResampleDconnMemorySurfacePostdilateParameters = typing.TypedDict('CiftiResampleDconnMemorySurfacePostdilateParameters', {
    "@type": typing.NotRequired[typing.Literal["surface_postdilate"]],
    "dilate_mm": float,
    "opt_nearest": bool,
    "opt_linear": bool,
    "weighted": typing.NotRequired[CiftiResampleDconnMemoryWeighted1Parameters | None],
})
CiftiResampleDconnMemorySurfacePostdilateParametersTagged = typing.TypedDict('CiftiResampleDconnMemorySurfacePostdilateParametersTagged', {
    "@type": typing.Literal["surface_postdilate"],
    "dilate_mm": float,
    "opt_nearest": bool,
    "opt_linear": bool,
    "weighted": typing.NotRequired[CiftiResampleDconnMemoryWeighted1Parameters | None],
})


CiftiResampleDconnMemoryFlirtParameters = typing.TypedDict('CiftiResampleDconnMemoryFlirtParameters', {
    "@type": typing.NotRequired[typing.Literal["flirt"]],
    "source_volume": str,
    "target_volume": str,
})
CiftiResampleDconnMemoryFlirtParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryFlirtParametersTagged', {
    "@type": typing.Literal["flirt"],
    "source_volume": str,
    "target_volume": str,
})


CiftiResampleDconnMemoryAffineParameters = typing.TypedDict('CiftiResampleDconnMemoryAffineParameters', {
    "@type": typing.NotRequired[typing.Literal["affine"]],
    "affine_file": str,
    "flirt": typing.NotRequired[CiftiResampleDconnMemoryFlirtParameters | None],
})
CiftiResampleDconnMemoryAffineParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryAffineParametersTagged', {
    "@type": typing.Literal["affine"],
    "affine_file": str,
    "flirt": typing.NotRequired[CiftiResampleDconnMemoryFlirtParameters | None],
})


CiftiResampleDconnMemoryWarpfieldParameters = typing.TypedDict('CiftiResampleDconnMemoryWarpfieldParameters', {
    "@type": typing.NotRequired[typing.Literal["warpfield"]],
    "warpfield": str,
    "opt_fnirt_source_volume": typing.NotRequired[str | None],
})
CiftiResampleDconnMemoryWarpfieldParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryWarpfieldParametersTagged', {
    "@type": typing.Literal["warpfield"],
    "warpfield": str,
    "opt_fnirt_source_volume": typing.NotRequired[str | None],
})


CiftiResampleDconnMemoryLeftAreaSurfsParameters = typing.TypedDict('CiftiResampleDconnMemoryLeftAreaSurfsParameters', {
    "@type": typing.NotRequired[typing.Literal["left_area_surfs"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryLeftAreaSurfsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryLeftAreaSurfsParametersTagged', {
    "@type": typing.Literal["left_area_surfs"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryLeftAreaMetricsParameters = typing.TypedDict('CiftiResampleDconnMemoryLeftAreaMetricsParameters', {
    "@type": typing.NotRequired[typing.Literal["left_area_metrics"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryLeftAreaMetricsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryLeftAreaMetricsParametersTagged', {
    "@type": typing.Literal["left_area_metrics"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryLeftSpheresParameters = typing.TypedDict('CiftiResampleDconnMemoryLeftSpheresParameters', {
    "@type": typing.NotRequired[typing.Literal["left_spheres"]],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "left_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryLeftAreaSurfsParameters | None],
    "left_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryLeftAreaMetricsParameters | None],
})
CiftiResampleDconnMemoryLeftSpheresParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryLeftSpheresParametersTagged', {
    "@type": typing.Literal["left_spheres"],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "left_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryLeftAreaSurfsParameters | None],
    "left_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryLeftAreaMetricsParameters | None],
})


CiftiResampleDconnMemoryRightAreaSurfsParameters = typing.TypedDict('CiftiResampleDconnMemoryRightAreaSurfsParameters', {
    "@type": typing.NotRequired[typing.Literal["right_area_surfs"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryRightAreaSurfsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryRightAreaSurfsParametersTagged', {
    "@type": typing.Literal["right_area_surfs"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryRightAreaMetricsParameters = typing.TypedDict('CiftiResampleDconnMemoryRightAreaMetricsParameters', {
    "@type": typing.NotRequired[typing.Literal["right_area_metrics"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryRightAreaMetricsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryRightAreaMetricsParametersTagged', {
    "@type": typing.Literal["right_area_metrics"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryRightSpheresParameters = typing.TypedDict('CiftiResampleDconnMemoryRightSpheresParameters', {
    "@type": typing.NotRequired[typing.Literal["right_spheres"]],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "right_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryRightAreaSurfsParameters | None],
    "right_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryRightAreaMetricsParameters | None],
})
CiftiResampleDconnMemoryRightSpheresParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryRightSpheresParametersTagged', {
    "@type": typing.Literal["right_spheres"],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "right_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryRightAreaSurfsParameters | None],
    "right_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryRightAreaMetricsParameters | None],
})


CiftiResampleDconnMemoryCerebellumAreaSurfsParameters = typing.TypedDict('CiftiResampleDconnMemoryCerebellumAreaSurfsParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_area_surfs"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryCerebellumAreaSurfsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryCerebellumAreaSurfsParametersTagged', {
    "@type": typing.Literal["cerebellum_area_surfs"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryCerebellumAreaMetricsParameters = typing.TypedDict('CiftiResampleDconnMemoryCerebellumAreaMetricsParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_area_metrics"]],
    "current_area": InputPathType,
    "new_area": InputPathType,
})
CiftiResampleDconnMemoryCerebellumAreaMetricsParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryCerebellumAreaMetricsParametersTagged', {
    "@type": typing.Literal["cerebellum_area_metrics"],
    "current_area": InputPathType,
    "new_area": InputPathType,
})


CiftiResampleDconnMemoryCerebellumSpheresParameters = typing.TypedDict('CiftiResampleDconnMemoryCerebellumSpheresParameters', {
    "@type": typing.NotRequired[typing.Literal["cerebellum_spheres"]],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "cerebellum_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryCerebellumAreaSurfsParameters | None],
    "cerebellum_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryCerebellumAreaMetricsParameters | None],
})
CiftiResampleDconnMemoryCerebellumSpheresParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryCerebellumSpheresParametersTagged', {
    "@type": typing.Literal["cerebellum_spheres"],
    "current_sphere": InputPathType,
    "new_sphere": InputPathType,
    "cerebellum_area_surfs": typing.NotRequired[CiftiResampleDconnMemoryCerebellumAreaSurfsParameters | None],
    "cerebellum_area_metrics": typing.NotRequired[CiftiResampleDconnMemoryCerebellumAreaMetricsParameters | None],
})


CiftiResampleDconnMemoryParameters = typing.TypedDict('CiftiResampleDconnMemoryParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-resample-dconn-memory"]],
    "cifti_in": InputPathType,
    "cifti_template": InputPathType,
    "template_direction": str,
    "surface_method": str,
    "volume_method": str,
    "cifti_out": str,
    "opt_surface_largest": bool,
    "volume_predilate": typing.NotRequired[CiftiResampleDconnMemoryVolumePredilateParameters | None],
    "surface_postdilate": typing.NotRequired[CiftiResampleDconnMemorySurfacePostdilateParameters | None],
    "affine": typing.NotRequired[CiftiResampleDconnMemoryAffineParameters | None],
    "warpfield": typing.NotRequired[CiftiResampleDconnMemoryWarpfieldParameters | None],
    "left_spheres": typing.NotRequired[CiftiResampleDconnMemoryLeftSpheresParameters | None],
    "right_spheres": typing.NotRequired[CiftiResampleDconnMemoryRightSpheresParameters | None],
    "cerebellum_spheres": typing.NotRequired[CiftiResampleDconnMemoryCerebellumSpheresParameters | None],
})
CiftiResampleDconnMemoryParametersTagged = typing.TypedDict('CiftiResampleDconnMemoryParametersTagged', {
    "@type": typing.Literal["workbench/cifti-resample-dconn-memory"],
    "cifti_in": InputPathType,
    "cifti_template": InputPathType,
    "template_direction": str,
    "surface_method": str,
    "volume_method": str,
    "cifti_out": str,
    "opt_surface_largest": bool,
    "volume_predilate": typing.NotRequired[CiftiResampleDconnMemoryVolumePredilateParameters | None],
    "surface_postdilate": typing.NotRequired[CiftiResampleDconnMemorySurfacePostdilateParameters | None],
    "affine": typing.NotRequired[CiftiResampleDconnMemoryAffineParameters | None],
    "warpfield": typing.NotRequired[CiftiResampleDconnMemoryWarpfieldParameters | None],
    "left_spheres": typing.NotRequired[CiftiResampleDconnMemoryLeftSpheresParameters | None],
    "right_spheres": typing.NotRequired[CiftiResampleDconnMemoryRightSpheresParameters | None],
    "cerebellum_spheres": typing.NotRequired[CiftiResampleDconnMemoryCerebellumSpheresParameters | None],
})


def cifti_resample_dconn_memory_weighted_params(
    opt_exponent_exponent: float | None = None,
    opt_legacy_cutoff: bool = False,
) -> CiftiResampleDconnMemoryWeightedParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_exponent_exponent: specify exponent in weighting function: exponent\
            'n' to use in (1 / (distance ^ n)) as the weighting function (default\
            7).
        opt_legacy_cutoff: use v1.3.2 logic for the kernel cutoff.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weighted",
        "opt_legacy_cutoff": opt_legacy_cutoff,
    }
    if opt_exponent_exponent is not None:
        params["opt_exponent_exponent"] = opt_exponent_exponent
    return params


def cifti_resample_dconn_memory_weighted_cargs(
    params: CiftiResampleDconnMemoryWeightedParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-weighted")
    if params.get("opt_exponent_exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("opt_exponent_exponent", None))
        ])
    if params.get("opt_legacy_cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def cifti_resample_dconn_memory_volume_predilate_params(
    dilate_mm: float,
    opt_nearest: bool = False,
    weighted: CiftiResampleDconnMemoryWeightedParameters | None = None,
) -> CiftiResampleDconnMemoryVolumePredilateParametersTagged:
    """
    Build parameters.
    
    Args:
        dilate_mm: distance, in mm, to dilate.
        opt_nearest: use nearest value dilation.
        weighted: use weighted dilation (default).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume_predilate",
        "dilate_mm": dilate_mm,
        "opt_nearest": opt_nearest,
    }
    if weighted is not None:
        params["weighted"] = weighted
    return params


def cifti_resample_dconn_memory_volume_predilate_cargs(
    params: CiftiResampleDconnMemoryVolumePredilateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-volume-predilate")
    cargs.append(str(params.get("dilate_mm", None)))
    if params.get("opt_nearest", False):
        cargs.append("-nearest")
    if params.get("weighted", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_weighted_cargs(params.get("weighted", None), execution))
    return cargs


def cifti_resample_dconn_memory_weighted_1_params(
    opt_exponent_exponent: float | None = None,
    opt_legacy_cutoff: bool = False,
) -> CiftiResampleDconnMemoryWeighted1ParametersTagged:
    """
    Build parameters.
    
    Args:
        opt_exponent_exponent: specify exponent in weighting function: exponent\
            'n' to use in (area / (distance ^ n)) as the weighting function\
            (default 6).
        opt_legacy_cutoff: use v1.3.2 logic for the kernel cutoff.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weighted_1",
        "opt_legacy_cutoff": opt_legacy_cutoff,
    }
    if opt_exponent_exponent is not None:
        params["opt_exponent_exponent"] = opt_exponent_exponent
    return params


def cifti_resample_dconn_memory_weighted_1_cargs(
    params: CiftiResampleDconnMemoryWeighted1Parameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-weighted")
    if params.get("opt_exponent_exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("opt_exponent_exponent", None))
        ])
    if params.get("opt_legacy_cutoff", False):
        cargs.append("-legacy-cutoff")
    return cargs


def cifti_resample_dconn_memory_surface_postdilate_params(
    dilate_mm: float,
    opt_nearest: bool = False,
    opt_linear: bool = False,
    weighted: CiftiResampleDconnMemoryWeighted1Parameters | None = None,
) -> CiftiResampleDconnMemorySurfacePostdilateParametersTagged:
    """
    Build parameters.
    
    Args:
        dilate_mm: distance, in mm, to dilate.
        opt_nearest: use nearest value dilation.
        opt_linear: use linear dilation.
        weighted: use weighted dilation (default).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface_postdilate",
        "dilate_mm": dilate_mm,
        "opt_nearest": opt_nearest,
        "opt_linear": opt_linear,
    }
    if weighted is not None:
        params["weighted"] = weighted
    return params


def cifti_resample_dconn_memory_surface_postdilate_cargs(
    params: CiftiResampleDconnMemorySurfacePostdilateParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-surface-postdilate")
    cargs.append(str(params.get("dilate_mm", None)))
    if params.get("opt_nearest", False):
        cargs.append("-nearest")
    if params.get("opt_linear", False):
        cargs.append("-linear")
    if params.get("weighted", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_weighted_1_cargs(params.get("weighted", None), execution))
    return cargs


def cifti_resample_dconn_memory_flirt_params(
    source_volume: str,
    target_volume: str,
) -> CiftiResampleDconnMemoryFlirtParametersTagged:
    """
    Build parameters.
    
    Args:
        source_volume: the source volume used when generating the affine.
        target_volume: the target volume used when generating the affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt",
        "source_volume": source_volume,
        "target_volume": target_volume,
    }
    return params


def cifti_resample_dconn_memory_flirt_cargs(
    params: CiftiResampleDconnMemoryFlirtParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-flirt")
    cargs.append(params.get("source_volume", None))
    cargs.append(params.get("target_volume", None))
    return cargs


def cifti_resample_dconn_memory_affine_params(
    affine_file: str,
    flirt: CiftiResampleDconnMemoryFlirtParameters | None = None,
) -> CiftiResampleDconnMemoryAffineParametersTagged:
    """
    Build parameters.
    
    Args:
        affine_file: the affine file to use.
        flirt: MUST be used if affine is a flirt affine.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "affine",
        "affine_file": affine_file,
    }
    if flirt is not None:
        params["flirt"] = flirt
    return params


def cifti_resample_dconn_memory_affine_cargs(
    params: CiftiResampleDconnMemoryAffineParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-affine")
    cargs.append(params.get("affine_file", None))
    if params.get("flirt", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_flirt_cargs(params.get("flirt", None), execution))
    return cargs


def cifti_resample_dconn_memory_warpfield_params(
    warpfield: str,
    opt_fnirt_source_volume: str | None = None,
) -> CiftiResampleDconnMemoryWarpfieldParametersTagged:
    """
    Build parameters.
    
    Args:
        warpfield: the warpfield to use.
        opt_fnirt_source_volume: MUST be used if using a fnirt warpfield: the\
            source volume used when generating the warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "warpfield",
        "warpfield": warpfield,
    }
    if opt_fnirt_source_volume is not None:
        params["opt_fnirt_source_volume"] = opt_fnirt_source_volume
    return params


def cifti_resample_dconn_memory_warpfield_cargs(
    params: CiftiResampleDconnMemoryWarpfieldParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-warpfield")
    cargs.append(params.get("warpfield", None))
    if params.get("opt_fnirt_source_volume", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("opt_fnirt_source_volume", None)
        ])
    return cargs


def cifti_resample_dconn_memory_left_area_surfs_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryLeftAreaSurfsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant left anatomical surface with current mesh.
        new_area: a relevant left anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_area_surfs",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_left_area_surfs_cargs(
    params: CiftiResampleDconnMemoryLeftAreaSurfsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-area-surfs")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_left_area_metrics_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryLeftAreaMetricsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_area_metrics",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_left_area_metrics_cargs(
    params: CiftiResampleDconnMemoryLeftAreaMetricsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-area-metrics")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_left_spheres_params(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    left_area_surfs: CiftiResampleDconnMemoryLeftAreaSurfsParameters | None = None,
    left_area_metrics: CiftiResampleDconnMemoryLeftAreaMetricsParameters | None = None,
) -> CiftiResampleDconnMemoryLeftSpheresParametersTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current left surface.
        new_sphere: a sphere with the new left mesh that is in register with\
            the current sphere.
        left_area_surfs: specify left surfaces to do vertex area correction\
            based on.
        left_area_metrics: specify left vertex area metrics to do area\
            correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left_spheres",
        "current_sphere": current_sphere,
        "new_sphere": new_sphere,
    }
    if left_area_surfs is not None:
        params["left_area_surfs"] = left_area_surfs
    if left_area_metrics is not None:
        params["left_area_metrics"] = left_area_metrics
    return params


def cifti_resample_dconn_memory_left_spheres_cargs(
    params: CiftiResampleDconnMemoryLeftSpheresParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-left-spheres")
    cargs.append(execution.input_file(params.get("current_sphere", None)))
    cargs.append(execution.input_file(params.get("new_sphere", None)))
    if params.get("left_area_surfs", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_left_area_surfs_cargs(params.get("left_area_surfs", None), execution))
    if params.get("left_area_metrics", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_left_area_metrics_cargs(params.get("left_area_metrics", None), execution))
    return cargs


def cifti_resample_dconn_memory_right_area_surfs_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryRightAreaSurfsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant right anatomical surface with current mesh.
        new_area: a relevant right anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_area_surfs",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_right_area_surfs_cargs(
    params: CiftiResampleDconnMemoryRightAreaSurfsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-area-surfs")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_right_area_metrics_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryRightAreaMetricsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_area_metrics",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_right_area_metrics_cargs(
    params: CiftiResampleDconnMemoryRightAreaMetricsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-area-metrics")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_right_spheres_params(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    right_area_surfs: CiftiResampleDconnMemoryRightAreaSurfsParameters | None = None,
    right_area_metrics: CiftiResampleDconnMemoryRightAreaMetricsParameters | None = None,
) -> CiftiResampleDconnMemoryRightSpheresParametersTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current right\
            surface.
        new_sphere: a sphere with the new right mesh that is in register with\
            the current sphere.
        right_area_surfs: specify right surfaces to do vertex area correction\
            based on.
        right_area_metrics: specify right vertex area metrics to do area\
            correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right_spheres",
        "current_sphere": current_sphere,
        "new_sphere": new_sphere,
    }
    if right_area_surfs is not None:
        params["right_area_surfs"] = right_area_surfs
    if right_area_metrics is not None:
        params["right_area_metrics"] = right_area_metrics
    return params


def cifti_resample_dconn_memory_right_spheres_cargs(
    params: CiftiResampleDconnMemoryRightSpheresParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-right-spheres")
    cargs.append(execution.input_file(params.get("current_sphere", None)))
    cargs.append(execution.input_file(params.get("new_sphere", None)))
    if params.get("right_area_surfs", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_right_area_surfs_cargs(params.get("right_area_surfs", None), execution))
    if params.get("right_area_metrics", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_right_area_metrics_cargs(params.get("right_area_metrics", None), execution))
    return cargs


def cifti_resample_dconn_memory_cerebellum_area_surfs_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryCerebellumAreaSurfsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a relevant cerebellum anatomical surface with current\
            mesh.
        new_area: a relevant cerebellum anatomical surface with new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_area_surfs",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_cerebellum_area_surfs_cargs(
    params: CiftiResampleDconnMemoryCerebellumAreaSurfsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-area-surfs")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_cerebellum_area_metrics_params(
    current_area: InputPathType,
    new_area: InputPathType,
) -> CiftiResampleDconnMemoryCerebellumAreaMetricsParametersTagged:
    """
    Build parameters.
    
    Args:
        current_area: a metric file with vertex areas for the current mesh.
        new_area: a metric file with vertex areas for the new mesh.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_area_metrics",
        "current_area": current_area,
        "new_area": new_area,
    }
    return params


def cifti_resample_dconn_memory_cerebellum_area_metrics_cargs(
    params: CiftiResampleDconnMemoryCerebellumAreaMetricsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-area-metrics")
    cargs.append(execution.input_file(params.get("current_area", None)))
    cargs.append(execution.input_file(params.get("new_area", None)))
    return cargs


def cifti_resample_dconn_memory_cerebellum_spheres_params(
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    cerebellum_area_surfs: CiftiResampleDconnMemoryCerebellumAreaSurfsParameters | None = None,
    cerebellum_area_metrics: CiftiResampleDconnMemoryCerebellumAreaMetricsParameters | None = None,
) -> CiftiResampleDconnMemoryCerebellumSpheresParametersTagged:
    """
    Build parameters.
    
    Args:
        current_sphere: a sphere with the same mesh as the current cerebellum\
            surface.
        new_sphere: a sphere with the new cerebellum mesh that is in register\
            with the current sphere.
        cerebellum_area_surfs: specify cerebellum surfaces to do vertex area\
            correction based on.
        cerebellum_area_metrics: specify cerebellum vertex area metrics to do\
            area correction based on.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum_spheres",
        "current_sphere": current_sphere,
        "new_sphere": new_sphere,
    }
    if cerebellum_area_surfs is not None:
        params["cerebellum_area_surfs"] = cerebellum_area_surfs
    if cerebellum_area_metrics is not None:
        params["cerebellum_area_metrics"] = cerebellum_area_metrics
    return params


def cifti_resample_dconn_memory_cerebellum_spheres_cargs(
    params: CiftiResampleDconnMemoryCerebellumSpheresParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cerebellum-spheres")
    cargs.append(execution.input_file(params.get("current_sphere", None)))
    cargs.append(execution.input_file(params.get("new_sphere", None)))
    if params.get("cerebellum_area_surfs", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_cerebellum_area_surfs_cargs(params.get("cerebellum_area_surfs", None), execution))
    if params.get("cerebellum_area_metrics", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_cerebellum_area_metrics_cargs(params.get("cerebellum_area_metrics", None), execution))
    return cargs


class CiftiResampleDconnMemoryOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiResampleDconnMemoryParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_resample_dconn_memory_params(
    cifti_in: InputPathType,
    cifti_template: InputPathType,
    template_direction: str,
    surface_method: str,
    volume_method: str,
    cifti_out: str,
    opt_surface_largest: bool = False,
    volume_predilate: CiftiResampleDconnMemoryVolumePredilateParameters | None = None,
    surface_postdilate: CiftiResampleDconnMemorySurfacePostdilateParameters | None = None,
    affine: CiftiResampleDconnMemoryAffineParameters | None = None,
    warpfield: CiftiResampleDconnMemoryWarpfieldParameters | None = None,
    left_spheres: CiftiResampleDconnMemoryLeftSpheresParameters | None = None,
    right_spheres: CiftiResampleDconnMemoryRightSpheresParameters | None = None,
    cerebellum_spheres: CiftiResampleDconnMemoryCerebellumSpheresParameters | None = None,
) -> CiftiResampleDconnMemoryParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to resample.
        cifti_template: a cifti file containing the cifti space to resample to.
        template_direction: the direction of the template to use as the\
            resampling space, ROW or COLUMN.
        surface_method: specify a surface resampling method.
        volume_method: specify a volume interpolation method.
        cifti_out: the output cifti file.
        opt_surface_largest: use largest weight instead of weighted average\
            when doing surface resampling.
        volume_predilate: dilate the volume components before resampling.
        surface_postdilate: dilate the surface components after resampling.
        affine: use an affine transformation on the volume components.
        warpfield: use a warpfield on the volume components.
        left_spheres: specify spheres for left surface resampling.
        right_spheres: specify spheres for right surface resampling.
        cerebellum_spheres: specify spheres for cerebellum surface resampling.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-resample-dconn-memory",
        "cifti_in": cifti_in,
        "cifti_template": cifti_template,
        "template_direction": template_direction,
        "surface_method": surface_method,
        "volume_method": volume_method,
        "cifti_out": cifti_out,
        "opt_surface_largest": opt_surface_largest,
    }
    if volume_predilate is not None:
        params["volume_predilate"] = volume_predilate
    if surface_postdilate is not None:
        params["surface_postdilate"] = surface_postdilate
    if affine is not None:
        params["affine"] = affine
    if warpfield is not None:
        params["warpfield"] = warpfield
    if left_spheres is not None:
        params["left_spheres"] = left_spheres
    if right_spheres is not None:
        params["right_spheres"] = right_spheres
    if cerebellum_spheres is not None:
        params["cerebellum_spheres"] = cerebellum_spheres
    return params


def cifti_resample_dconn_memory_cargs(
    params: CiftiResampleDconnMemoryParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-resample-dconn-memory")
    cargs.append(execution.input_file(params.get("cifti_in", None)))
    cargs.append(execution.input_file(params.get("cifti_template", None)))
    cargs.append(params.get("template_direction", None))
    cargs.append(params.get("surface_method", None))
    cargs.append(params.get("volume_method", None))
    cargs.append(params.get("cifti_out", None))
    if params.get("opt_surface_largest", False):
        cargs.append("-surface-largest")
    if params.get("volume_predilate", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_volume_predilate_cargs(params.get("volume_predilate", None), execution))
    if params.get("surface_postdilate", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_surface_postdilate_cargs(params.get("surface_postdilate", None), execution))
    if params.get("affine", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_affine_cargs(params.get("affine", None), execution))
    if params.get("warpfield", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_warpfield_cargs(params.get("warpfield", None), execution))
    if params.get("left_spheres", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_left_spheres_cargs(params.get("left_spheres", None), execution))
    if params.get("right_spheres", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_right_spheres_cargs(params.get("right_spheres", None), execution))
    if params.get("cerebellum_spheres", None) is not None:
        cargs.extend(cifti_resample_dconn_memory_cerebellum_spheres_cargs(params.get("cerebellum_spheres", None), execution))
    return cargs


def cifti_resample_dconn_memory_outputs(
    params: CiftiResampleDconnMemoryParameters,
    execution: Execution,
) -> CiftiResampleDconnMemoryOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiResampleDconnMemoryOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_resample_dconn_memory_execute(
    params: CiftiResampleDconnMemoryParameters,
    runner: Runner | None = None,
) -> CiftiResampleDconnMemoryOutputs:
    """
    cifti-resample-dconn-memory
    
    Use lots of memory to resample dconn.
    
    This command does the same thing as running -cifti-resample twice, but uses
    memory up to approximately 2x the size that the intermediate file would be.
    This is because the intermediate dconn is kept in memory, rather than
    written to disk, and the components before and after resampling/dilation
    have to be in memory at the same time during the relevant computation. The
    <template-direction> argument should usually be COLUMN, as dtseries,
    dscalar, and dlabel all have brainordinates on that direction. If spheres
    are not specified for a surface structure which exists in the cifti files,
    its data is copied without resampling or dilation. Dilation is done with the
    'nearest' method, and is done on <new-sphere> for surface data. Volume
    components are padded before dilation so that dilation doesn't run into the
    edge of the component bounding box.
    
    To get the v1.3.2 and earlier behavior of weighted dilation, specify
    exponent of 2 for surface and volume, and -legacy-cutoff for both surface
    and volume.
    
    The <volume-method> argument must be one of the following:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR
    
    The <surface-method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiResampleDconnMemoryOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_RESAMPLE_DCONN_MEMORY_METADATA)
    params = execution.params(params)
    cargs = cifti_resample_dconn_memory_cargs(params, execution)
    ret = cifti_resample_dconn_memory_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_resample_dconn_memory(
    cifti_in: InputPathType,
    cifti_template: InputPathType,
    template_direction: str,
    surface_method: str,
    volume_method: str,
    cifti_out: str,
    opt_surface_largest: bool = False,
    volume_predilate: CiftiResampleDconnMemoryVolumePredilateParameters | None = None,
    surface_postdilate: CiftiResampleDconnMemorySurfacePostdilateParameters | None = None,
    affine: CiftiResampleDconnMemoryAffineParameters | None = None,
    warpfield: CiftiResampleDconnMemoryWarpfieldParameters | None = None,
    left_spheres: CiftiResampleDconnMemoryLeftSpheresParameters | None = None,
    right_spheres: CiftiResampleDconnMemoryRightSpheresParameters | None = None,
    cerebellum_spheres: CiftiResampleDconnMemoryCerebellumSpheresParameters | None = None,
    runner: Runner | None = None,
) -> CiftiResampleDconnMemoryOutputs:
    """
    cifti-resample-dconn-memory
    
    Use lots of memory to resample dconn.
    
    This command does the same thing as running -cifti-resample twice, but uses
    memory up to approximately 2x the size that the intermediate file would be.
    This is because the intermediate dconn is kept in memory, rather than
    written to disk, and the components before and after resampling/dilation
    have to be in memory at the same time during the relevant computation. The
    <template-direction> argument should usually be COLUMN, as dtseries,
    dscalar, and dlabel all have brainordinates on that direction. If spheres
    are not specified for a surface structure which exists in the cifti files,
    its data is copied without resampling or dilation. Dilation is done with the
    'nearest' method, and is done on <new-sphere> for surface data. Volume
    components are padded before dilation so that dilation doesn't run into the
    edge of the component bounding box.
    
    To get the v1.3.2 and earlier behavior of weighted dilation, specify
    exponent of 2 for surface and volume, and -legacy-cutoff for both surface
    and volume.
    
    The <volume-method> argument must be one of the following:
    
    CUBIC
    ENCLOSING_VOXEL
    TRILINEAR
    
    The <surface-method> argument must be one of the following:
    
    ADAP_BARY_AREA
    BARYCENTRIC
    .
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_in: the cifti file to resample.
        cifti_template: a cifti file containing the cifti space to resample to.
        template_direction: the direction of the template to use as the\
            resampling space, ROW or COLUMN.
        surface_method: specify a surface resampling method.
        volume_method: specify a volume interpolation method.
        cifti_out: the output cifti file.
        opt_surface_largest: use largest weight instead of weighted average\
            when doing surface resampling.
        volume_predilate: dilate the volume components before resampling.
        surface_postdilate: dilate the surface components after resampling.
        affine: use an affine transformation on the volume components.
        warpfield: use a warpfield on the volume components.
        left_spheres: specify spheres for left surface resampling.
        right_spheres: specify spheres for right surface resampling.
        cerebellum_spheres: specify spheres for cerebellum surface resampling.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiResampleDconnMemoryOutputs`).
    """
    params = cifti_resample_dconn_memory_params(
        cifti_in=cifti_in,
        cifti_template=cifti_template,
        template_direction=template_direction,
        surface_method=surface_method,
        volume_method=volume_method,
        cifti_out=cifti_out,
        opt_surface_largest=opt_surface_largest,
        volume_predilate=volume_predilate,
        surface_postdilate=surface_postdilate,
        affine=affine,
        warpfield=warpfield,
        left_spheres=left_spheres,
        right_spheres=right_spheres,
        cerebellum_spheres=cerebellum_spheres,
    )
    return cifti_resample_dconn_memory_execute(params, runner)


__all__ = [
    "CIFTI_RESAMPLE_DCONN_MEMORY_METADATA",
    "CiftiResampleDconnMemoryOutputs",
    "cifti_resample_dconn_memory",
    "cifti_resample_dconn_memory_affine_params",
    "cifti_resample_dconn_memory_cerebellum_area_metrics_params",
    "cifti_resample_dconn_memory_cerebellum_area_surfs_params",
    "cifti_resample_dconn_memory_cerebellum_spheres_params",
    "cifti_resample_dconn_memory_execute",
    "cifti_resample_dconn_memory_flirt_params",
    "cifti_resample_dconn_memory_left_area_metrics_params",
    "cifti_resample_dconn_memory_left_area_surfs_params",
    "cifti_resample_dconn_memory_left_spheres_params",
    "cifti_resample_dconn_memory_params",
    "cifti_resample_dconn_memory_right_area_metrics_params",
    "cifti_resample_dconn_memory_right_area_surfs_params",
    "cifti_resample_dconn_memory_right_spheres_params",
    "cifti_resample_dconn_memory_surface_postdilate_params",
    "cifti_resample_dconn_memory_volume_predilate_params",
    "cifti_resample_dconn_memory_warpfield_params",
    "cifti_resample_dconn_memory_weighted_1_params",
    "cifti_resample_dconn_memory_weighted_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_AVERAGE_METADATA = Metadata(
    id="e097bdbc9855ceeaf5b5317f6d524a83688c3dbe.boutiques",
    name="cifti-average",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


CiftiAverageExcludeOutliersParameters = typing.TypedDict('CiftiAverageExcludeOutliersParameters', {
    "@type": typing.NotRequired[typing.Literal["exclude_outliers"]],
    "sigma_below": float,
    "sigma_above": float,
})
CiftiAverageExcludeOutliersParametersTagged = typing.TypedDict('CiftiAverageExcludeOutliersParametersTagged', {
    "@type": typing.Literal["exclude_outliers"],
    "sigma_below": float,
    "sigma_above": float,
})


CiftiAverageCiftiParameters = typing.TypedDict('CiftiAverageCiftiParameters', {
    "@type": typing.NotRequired[typing.Literal["cifti"]],
    "cifti_in": InputPathType,
    "opt_weight_weight": typing.NotRequired[float | None],
})
CiftiAverageCiftiParametersTagged = typing.TypedDict('CiftiAverageCiftiParametersTagged', {
    "@type": typing.Literal["cifti"],
    "cifti_in": InputPathType,
    "opt_weight_weight": typing.NotRequired[float | None],
})


CiftiAverageParameters = typing.TypedDict('CiftiAverageParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/cifti-average"]],
    "cifti_out": str,
    "exclude_outliers": typing.NotRequired[CiftiAverageExcludeOutliersParameters | None],
    "opt_mem_limit_limit_gb": typing.NotRequired[float | None],
    "cifti": typing.NotRequired[list[CiftiAverageCiftiParameters] | None],
})
CiftiAverageParametersTagged = typing.TypedDict('CiftiAverageParametersTagged', {
    "@type": typing.Literal["workbench/cifti-average"],
    "cifti_out": str,
    "exclude_outliers": typing.NotRequired[CiftiAverageExcludeOutliersParameters | None],
    "opt_mem_limit_limit_gb": typing.NotRequired[float | None],
    "cifti": typing.NotRequired[list[CiftiAverageCiftiParameters] | None],
})


def cifti_average_exclude_outliers_params(
    sigma_below: float,
    sigma_above: float,
) -> CiftiAverageExcludeOutliersParametersTagged:
    """
    Build parameters.
    
    Args:
        sigma_below: number of standard deviations below the mean to include.
        sigma_above: number of standard deviations above the mean to include.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exclude_outliers",
        "sigma_below": sigma_below,
        "sigma_above": sigma_above,
    }
    return params


def cifti_average_exclude_outliers_cargs(
    params: CiftiAverageExcludeOutliersParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-exclude-outliers")
    cargs.append(str(params.get("sigma_below", None)))
    cargs.append(str(params.get("sigma_above", None)))
    return cargs


def cifti_average_cifti_params(
    cifti_in: InputPathType,
    opt_weight_weight: float | None = None,
) -> CiftiAverageCiftiParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        opt_weight_weight: give a weight for this file: the weight to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti_in": cifti_in,
    }
    if opt_weight_weight is not None:
        params["opt_weight_weight"] = opt_weight_weight
    return params


def cifti_average_cifti_cargs(
    params: CiftiAverageCiftiParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-cifti")
    cargs.append(execution.input_file(params.get("cifti_in", None)))
    if params.get("opt_weight_weight", None) is not None:
        cargs.extend([
            "-weight",
            str(params.get("opt_weight_weight", None))
        ])
    return cargs


class CiftiAverageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiAverageParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_average_params(
    cifti_out: str,
    exclude_outliers: CiftiAverageExcludeOutliersParameters | None = None,
    opt_mem_limit_limit_gb: float | None = None,
    cifti: list[CiftiAverageCiftiParameters] | None = None,
) -> CiftiAverageParametersTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: output cifti file.
        exclude_outliers: exclude outliers by standard deviation of each\
            element across files.
        opt_mem_limit_limit_gb: restrict memory used for file reading\
            efficiency: memory limit in gigabytes.
        cifti: specify an input file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-average",
        "cifti_out": cifti_out,
    }
    if exclude_outliers is not None:
        params["exclude_outliers"] = exclude_outliers
    if opt_mem_limit_limit_gb is not None:
        params["opt_mem_limit_limit_gb"] = opt_mem_limit_limit_gb
    if cifti is not None:
        params["cifti"] = cifti
    return params


def cifti_average_cargs(
    params: CiftiAverageParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-average")
    cargs.append(params.get("cifti_out", None))
    if params.get("exclude_outliers", None) is not None:
        cargs.extend(cifti_average_exclude_outliers_cargs(params.get("exclude_outliers", None), execution))
    if params.get("opt_mem_limit_limit_gb", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("opt_mem_limit_limit_gb", None))
        ])
    if params.get("cifti", None) is not None:
        cargs.extend([a for c in [cifti_average_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c])
    return cargs


def cifti_average_outputs(
    params: CiftiAverageParameters,
    execution: Execution,
) -> CiftiAverageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiAverageOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti_out", None)),
    )
    return ret


def cifti_average_execute(
    params: CiftiAverageParameters,
    runner: Runner | None = None,
) -> CiftiAverageOutputs:
    """
    cifti-average
    
    Average cifti files.
    
    Averages cifti files together. Files without -weight specified are given a
    weight of 1. If -exclude-outliers is specified, at each element, the data
    across all files is taken as a set, its unweighted mean and sample standard
    deviation are found, and values outside the specified number of standard
    deviations are excluded from the (potentially weighted) average at that
    element.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_AVERAGE_METADATA)
    params = execution.params(params)
    cargs = cifti_average_cargs(params, execution)
    ret = cifti_average_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_average(
    cifti_out: str,
    exclude_outliers: CiftiAverageExcludeOutliersParameters | None = None,
    opt_mem_limit_limit_gb: float | None = None,
    cifti: list[CiftiAverageCiftiParameters] | None = None,
    runner: Runner | None = None,
) -> CiftiAverageOutputs:
    """
    cifti-average
    
    Average cifti files.
    
    Averages cifti files together. Files without -weight specified are given a
    weight of 1. If -exclude-outliers is specified, at each element, the data
    across all files is taken as a set, its unweighted mean and sample standard
    deviation are found, and values outside the specified number of standard
    deviations are excluded from the (potentially weighted) average at that
    element.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_out: output cifti file.
        exclude_outliers: exclude outliers by standard deviation of each\
            element across files.
        opt_mem_limit_limit_gb: restrict memory used for file reading\
            efficiency: memory limit in gigabytes.
        cifti: specify an input file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageOutputs`).
    """
    params = cifti_average_params(
        cifti_out=cifti_out,
        exclude_outliers=exclude_outliers,
        opt_mem_limit_limit_gb=opt_mem_limit_limit_gb,
        cifti=cifti,
    )
    return cifti_average_execute(params, runner)


__all__ = [
    "CIFTI_AVERAGE_METADATA",
    "CiftiAverageOutputs",
    "cifti_average",
    "cifti_average_cifti_params",
    "cifti_average_exclude_outliers_params",
    "cifti_average_execute",
    "cifti_average_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BACKEND_AVERAGE_ROI_CORRELATION_METADATA = Metadata(
    id="9178c51e54512af98ce12ec548335b4ce7759c19.boutiques",
    name="backend-average-roi-correlation",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


BackendAverageRoiCorrelationParameters = typing.TypedDict('BackendAverageRoiCorrelationParameters', {
    "@type": typing.NotRequired[typing.Literal["workbench/backend-average-roi-correlation"]],
    "index_list": str,
    "out_file": str,
})
BackendAverageRoiCorrelationParametersTagged = typing.TypedDict('BackendAverageRoiCorrelationParametersTagged', {
    "@type": typing.Literal["workbench/backend-average-roi-correlation"],
    "index_list": str,
    "out_file": str,
})


class BackendAverageRoiCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BackendAverageRoiCorrelationParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def backend_average_roi_correlation_params(
    index_list: str,
    out_file: str,
) -> BackendAverageRoiCorrelationParametersTagged:
    """
    Build parameters.
    
    Args:
        index_list: comma separated list of cifti indexes to average and then\
            correlate.
        out_file: file to write the average row to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/backend-average-roi-correlation",
        "index_list": index_list,
        "out_file": out_file,
    }
    return params


def backend_average_roi_correlation_cargs(
    params: BackendAverageRoiCorrelationParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("wb_command")
    cargs.append("-backend-average-roi-correlation")
    cargs.append(params.get("index_list", None))
    cargs.append(params.get("out_file", None))
    return cargs


def backend_average_roi_correlation_outputs(
    params: BackendAverageRoiCorrelationParameters,
    execution: Execution,
) -> BackendAverageRoiCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BackendAverageRoiCorrelationOutputs(
        root=execution.output_file("."),
    )
    return ret


def backend_average_roi_correlation_execute(
    params: BackendAverageRoiCorrelationParameters,
    runner: Runner | None = None,
) -> BackendAverageRoiCorrelationOutputs:
    """
    backend-average-roi-correlation
    
    Connectome db backend command for cifti average roi correlation.
    
    This command is probably not the one you are looking for, try
    -cifti-average-roi-correlation. It takes the list of cifti files to average
    from standard input, and writes its output as little endian, 32-bit integer
    of row size followed by the row as 32-bit floats.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BackendAverageRoiCorrelationOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BACKEND_AVERAGE_ROI_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = backend_average_roi_correlation_cargs(params, execution)
    ret = backend_average_roi_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def backend_average_roi_correlation(
    index_list: str,
    out_file: str,
    runner: Runner | None = None,
) -> BackendAverageRoiCorrelationOutputs:
    """
    backend-average-roi-correlation
    
    Connectome db backend command for cifti average roi correlation.
    
    This command is probably not the one you are looking for, try
    -cifti-average-roi-correlation. It takes the list of cifti files to average
    from standard input, and writes its output as little endian, 32-bit integer
    of row size followed by the row as 32-bit floats.
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        index_list: comma separated list of cifti indexes to average and then\
            correlate.
        out_file: file to write the average row to.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BackendAverageRoiCorrelationOutputs`).
    """
    params = backend_average_roi_correlation_params(
        index_list=index_list,
        out_file=out_file,
    )
    return backend_average_roi_correlation_execute(params, runner)


__all__ = [
    "BACKEND_AVERAGE_ROI_CORRELATION_METADATA",
    "BackendAverageRoiCorrelationOutputs",
    "backend_average_roi_correlation",
    "backend_average_roi_correlation_execute",
    "backend_average_roi_correlation_params",
]
